import { MaterialAPI } from '@/config/API.ts'
import axiosInstance from '@/utils/axios/axiosInstance.ts'
import type {
  TagsRes,
  UploadAssetForm,
} from '@/views/Material/types/uploadFileType.ts'

import type {
  UploadFile,
  UploadFiles,
  UploadRawFile,
  UploadUserFile,
} from 'element-plus'
import { type Reactive, type Ref } from 'vue'

export function useFileUpload() {
  function isValidFile(file: UploadFile | UploadRawFile) {
    // 检查文件对象和 raw 是否存在
    if (!file) {
      return false
    }
    let resultFile: UploadRawFile | null = null
    if ('raw' in file) {
      resultFile = file['raw'] as UploadRawFile
    } else {
      resultFile = file as UploadRawFile
    }

    // 支持的文件类型
    const supportedTypes = ['image/png', 'image/jpeg', 'video/mp4', 'video/avi']

    // 最大文件大小：100MB
    const maxSize = 100 * 1024 * 1024 // 104,857,600 字节

    // 获取文件的类型和大小
    const fileType = resultFile.type
    const fileSize = resultFile.size

    // 检查类型是否支持且大小是否在限制内
    return supportedTypes.includes(fileType) && fileSize <= maxSize
  }

  function getFileCategory(fileType: string): 'img' | 'video' | 'unknown' {
    // 检查输入是否为字符串
    if (typeof fileType !== 'string') {
      console.error('fileType must be a string')
      return 'unknown'
    }

    // 判断文件类型
    if (fileType.startsWith('image/')) {
      return 'img'
    } else if (fileType.startsWith('video/')) {
      return 'video'
    } else {
      return 'unknown'
    }
  }

  const handleFile = (
    file: UploadFile | UploadRawFile,
    fileList: UploadUserFile[] | UploadFiles,
    isVideo: Ref<boolean>,
    formData: Reactive<UploadAssetForm>,
  ) => {
    console.log(JSON.parse(JSON.stringify(formData)))
    if (!isValidFile(file)) {
      fileList.splice(1)
      ElMessage.warning('上传文件超出限制或不是规定文件')
      return false
    }
    const fileType =
      'raw' in file
        ? (file as UploadFile).raw?.type
        : (file as UploadRawFile).type
    if (!fileType) {
      ElMessage.warning('请上传有效文件')
      return false
    }
    const type = getFileCategory(fileType)
    if (!type || type === 'unknown') {
      ElMessage.warning('文件类型无效')
      return
    }
    isVideo.value = type === 'video'
    // 仅用于过表单检测
    formData.filePath = file.name
    console.log(JSON.parse(JSON.stringify(formData)))
    // nextTick(() => {
    //   formRef.value?.validateField('filePath')
    // })
  }

  const getAllTags = async () => {
    const res = (await axiosInstance.post(
      MaterialAPI.getAllTags,
      {},
    )) as TagsRes
    if (res.code !== 0) {
      ElMessage.error('获取标签失败')
      return []
    }
    return res.data
  }

  const validateFile = (rule: any, value: any, callback: any) => {
    console.log(rule)
    console.log(value)
    if (value === '') {
      callback(new Error('Please input the password'))
    } else {
      // if (ruleForm.checkPass !== '') {
      //   if (!ruleFormRef.value) return
      //   ruleFormRef.value.validateField('checkPass')
      // }
      callback()
    }
  }

  return {
    isValidFile,
    getFileCategory,
    handleFile,
    getAllTags,
    validateFile,
  }
}
